
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include <GL/glew.h> // http://glew.sourceforge.net/basic.html

#include <GL/glfw.h> // http://www.glfw.org/GLFWReference27.pdf

//-------------- textures --------------
typedef unsigned char byte;

#pragma pack(push)
#pragma pack(1)

typedef struct
{
    byte  identsize;          // size of ID field that follows 18 byte header (0 usually)
    byte  colourmaptype;      // type of colour map 0=none, 1=has palette
    byte  imagetype;          // type of image 0=none,1=indexed,2=rgb,3=grey,+8=rle packed

    short colourmapstart;     // first colour map entry in palette
    short colourmaplength;    // number of colours in palette
    byte  colourmapbits;      // number of bits per palette entry 15,16,24,32

    short xstart;             // image x origin
    short ystart;             // image y origin
    short width;              // image width in pixels
    short height;             // image height in pixels
    byte  bpp;               // image bits per pixel 8,16,24,32
    byte  descriptor;         // image descriptor bits (vh flip bits)

    // pixel data follows header

} TGA_HEADER;

#pragma pack(pop)

GLvoid* loadTGA(const char* fname,TGA_HEADER& head)
{
	FILE* f=fopen(fname,"rb");
	if (!f)
    {
        fprintf(stderr,"Error opening TGAfile %s\n",fname);
        return 0;
    }
	fread(&head,1,sizeof(TGA_HEADER),f);

	GLvoid* pixels=0;
	if (head.imagetype==2) // RGB not packed
	{
	    pixels=(unsigned char*)malloc(head.width*head.height*head.bpp >> 3);
	    fread(pixels,head.height,head.width*(head.bpp>>3),f);
	}
	fclose(f);
	return pixels;
}

GLuint loadTGA(const char* fname)
{
	TGA_HEADER head;
	GLvoid* pixels=loadTGA(fname,head);
	GLuint texID=0;
	if (pixels) // RGB not packed
	{
	    glGenTextures(1,&texID);
	    glBindTexture(GL_TEXTURE_2D,texID);
	    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
	    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering

        switch (head.bpp)
        {
		    case 24 :
			    glTexImage2D(GL_TEXTURE_2D,0,GL_RGB8,head.width,head.height,0,GL_BGR,GL_UNSIGNED_BYTE,pixels);
		    break;
            case 32 :
			    glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA8,head.width,head.height,0,GL_BGRA,GL_UNSIGNED_BYTE,pixels);
		    break;
        }
	}
	free(pixels);
	return texID;
}

GLvoid* loadRaw(const char* fname,unsigned int x,unsigned int y,unsigned int z,unsigned int size_of_type)
{
    fprintf(stderr,"Loading RAW file %s ...",fname);
	FILE* f=fopen(fname,"rb");
	if (!f)
    {
        fprintf(stderr,"ERROR opening file\n",fname);
        return 0;
    }
	GLvoid* outData=new unsigned char[x*y*z*size_of_type];
	memset(outData,0,x*y*z*size_of_type);

	fseek(f,0,SEEK_END);
	unsigned int nSiz=ftell(f);
	fseek(f,0,SEEK_SET);

	fread(outData,nSiz,1,f);
	fclose(f);
    fprintf(stderr,"OK !\n");
	return outData;

}

GLuint loadRawTex3D(const char* fname)
{
	GLuint texID=0;

	GLuint x=256;
	GLuint y=256;
	GLuint z=256;

	GLvoid* voxels=loadRaw(fname,x,y,z,1);
	if (voxels)
	{
		glGenTextures(1,&texID);
		glBindTexture(GL_TEXTURE_3D,texID);
		glTexParameteri(GL_TEXTURE_3D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
		glTexParameteri(GL_TEXTURE_3D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering

		glTexParameteri(GL_TEXTURE_3D,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_3D,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
		glTexParameteri(GL_TEXTURE_3D,GL_TEXTURE_WRAP_R,GL_CLAMP_TO_EDGE);

		glPixelStorei(GL_UNPACK_ALIGNMENT,1);


		glTexImage3D(GL_TEXTURE_3D,0,GL_INTENSITY8,x,y,z,0,GL_LUMINANCE,GL_UNSIGNED_BYTE,voxels);
	}
	return texID;
}

GLuint loadTGACubeMap(const char* fname)
{
	const char* sidesuffix[]={"_posx.tga","_negx.tga","_posy.tga","_negy.tga","_posz.tga","_negz.tga"};
	char sidename[512];
	TGA_HEADER head;

	GLuint texID=0;
	glGenTextures(1,&texID);
	glBindTexture(GL_TEXTURE_CUBE_MAP,texID);

	glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
	glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_MAG_FILTER,GL_LINEAR);	// Linear Filtering
	glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
	glTexParameteri(GL_TEXTURE_CUBE_MAP,GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);

	glPixelStorei(GL_UNPACK_ALIGNMENT,1);

	for (int i=0;i<6;i++)
	{
		strcpy(sidename,fname);
		strcat(sidename,sidesuffix[i]);

		GLvoid* pixels=loadTGA(sidename,head);
		if (pixels != NULL)
		{
			switch (head.bpp)
			{
				case 24 :
					glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X+i,0,GL_RGB8,head.width,head.height,0,GL_BGR,GL_UNSIGNED_BYTE,pixels);
				break;
				case 32 :
					glTexImage2D(GL_TEXTURE_CUBE_MAP_POSITIVE_X+i,0,GL_RGBA8,head.width,head.height,0,GL_BGRA,GL_UNSIGNED_BYTE,pixels);
				break;
			}
			free(pixels);
		}
	}
	return texID;
}

void generateTransfer1D(GLuint texID)
{
	unsigned char table[256][4];
	for (int i=0;i<256;i++)
	{
		table[i][0]=i*2;
		table[i][1]=i;
		table[i][2]=i;
		if (i < 20)
			table[i][3]=0;
		else
			table[i][3]=i;
	};
	glBindTexture(GL_TEXTURE_1D,texID);
	glTexParameteri(GL_TEXTURE_1D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);	// Linear Filtering
	glPixelStorei(GL_UNPACK_ALIGNMENT,1);
	glTexImage1D(GL_TEXTURE_1D,0,GL_RGBA8,256,0,GL_RGBA,GL_UNSIGNED_BYTE,table);
//	glBindTexture(GL_TEXTURE_1D,0);
}